@@ -22,7 +22,7 @@
  * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  * SOFTWARE. */
 
-#include "options.h"
+#include "includes.h"
 #include "packet.h"
 #include "buffer.h"
 #include "session.h"
@@ -34,6 +34,8 @@
 #include "ssh.h"
 #include "random.h"
 #include "utmp.h"
+#include "x11fwd.h"
+#include "agentfwd.h"
 
 static int sessioncommand(struct Channel *channel, struct ChanSess *chansess,
 		char iscmd);
@@ -43,7 +45,6 @@ static int noptycommand(struct Channel *channel, struct ChanSess *chansess);
 static int ptycommand(struct Channel *channel, struct ChanSess *chansess);
 static int sessionwinchange(struct ChanSess *chansess);
 static void execchild(struct ChanSess *chansess);
-static void addnewvar(const char* param, const char* var);
 static void addchildpid(struct ChanSess *chansess, pid_t pid);
 static void sesssigchild_handler(int val);
 
@@ -198,7 +199,6 @@ void newchansess(struct Channel *channel) {
 	chansess->master = -1;
 	chansess->slave = -1;
 	chansess->tty = NULL;
-
 	chansess->term = NULL;
 	chansess->termw = 0;
 	chansess->termh = 0;
@@ -209,6 +209,18 @@ void newchansess(struct Channel *channel) {
 
 	channel->typedata = chansess;
 
+#ifndef DISABLE_X11FWD
+	chansess->x11fd = -1;
+	chansess->x11authprot = NULL;
+	chansess->x11authcookie = NULL;
+#endif
+
+#ifndef DISABLE_AGENTFWD
+	chansess->agentfd = -1;
+	chansess->agentfile = NULL;
+	chansess->agentdir = NULL;
+#endif
+
 }
 
 /* clean up a session type channel */
@@ -232,7 +244,7 @@ void closechansess(struct Channel *channel) {
 	if (chansess->tty) {
 		/* write the utmp/wtmp login record */
 		li = login_alloc_entry(chansess->pid, ses.authstate.username,
-				NULL, chansess->tty);
+				ses.hostname, chansess->tty);
 		login_logout(li);
 		login_free_entry(li);
 
@@ -240,6 +252,14 @@ void closechansess(struct Channel *channel) {
 		m_free(chansess->tty);
 	}
 
+#ifndef DISABLE_X11FWD
+	x11cleanup(chansess);
+#endif
+
+#ifndef DISABLE_AGENTFWD
+	agentcleanup(chansess);
+#endif
+
 	/* clear child pid entries */
 	for (i = 0; i < ses.childpidsize; i++) {
 		if (ses.childpids[i].chansess == chansess) {
@@ -282,18 +302,26 @@ void chansessionrequest(struct Channel *channel) {
 	assert(chansess != NULL);
 	TRACE(("type is %s\n", type));
 
-	if (strcmp(type, "exec") == 0) {
-		ret = sessioncommand(channel, chansess, 1);
+	if (strcmp(type, "window-change") == 0) {
+		ret = sessionwinchange(chansess);
 	} else if (strcmp(type, "shell") == 0) {
 		ret = sessioncommand(channel, chansess, 0);
 	} else if (strcmp(type, "pty-req") == 0) {
 		ret = sessionpty(chansess);
-	} else if (strcmp(type, "window-change") == 0) {
-		ret = sessionwinchange(chansess);
+	} else if (strcmp(type, "exec") == 0) {
+		ret = sessioncommand(channel, chansess, 1);
+#ifndef DISABLE_X11FWD
+	} else if (strcmp(type, "x11-req") == 0) {
+		ret = x11req(chansess);
+#endif
+#ifndef DISABLE_AGENTFWD
+	} else if (strcmp(type, "auth-agent-req@openssh.com") == 0) {
+		ret = agentreq(chansess);
+#endif
 	} else if (strcmp(type, "signal") == 0) {
 		ret = sessionsignal(chansess);
 	} else {
-		/* etc, todo "env", "subsystem", "x11-req" */
+		/* etc, todo "env", "subsystem" */
 	}
 
 	if (wantreply) {
@@ -388,12 +416,10 @@ static int sessionpty(struct ChanSess * chansess) {
 
 	/* allocate the pty */
 	assert(chansess->master == -1); /* haven't already got one */
-	TRACE(("about to pty_allocate"));
 	if (pty_allocate(&chansess->master, &chansess->slave, namebuf, 64) == 0) {
 		TRACE(("leave sessionpty: failed to allocate pty"));
 		return 1;
 	}
-	TRACE(("pty_allocate success (probably)"));
 	
 	chansess->tty = (char*)strdup(namebuf);
 	if (!chansess->tty) {
@@ -401,11 +427,8 @@ static int sessionpty(struct ChanSess * chansess) {
 	}
 
 	pty_setowner(ses.authstate.pw, chansess->tty);
-	TRACE(("done setowner"));
 	pty_change_window_size(chansess->master, chansess->termr, chansess->termc,
 			chansess->termw, chansess->termh);
-	TRACE(("done windowsize"));
-	
 
 	/* Term modes */
 	/* We'll ignore errors and continue if we can't set modes.
@@ -469,9 +492,7 @@ static int sessionpty(struct ChanSess * chansess) {
 			}
 		}
 		if (tcsetattr(chansess->master, TCSANOW, &termio) < 0) {
-			TRACE(("tcsetattr error"));
-		} else {
-			TRACE(("tcsetattr success"));
+			dropbear_log(LOG_INFO, "error setting terminal attributes");
 		}
 	}
 
@@ -540,24 +561,17 @@ static int noptycommand(struct Channel *channel, struct ChanSess *chansess) {
 		/* redirect stdin/stdout */
 #define FDIN 0
 #define FDOUT 1
-		if (dup2(infds[FDIN], STDIN_FILENO) < 0) {
-			TRACE(("leave sessioncommand: error redirecting stdin"));
+		if ((dup2(infds[FDIN], STDIN_FILENO) < 0) ||
+			(dup2(outfds[FDOUT], STDOUT_FILENO) < 0) ||
+			(dup2(errfds[FDOUT], STDERR_FILENO) < 0)) {
+			TRACE(("leave sessioncommand: error redirecting FDs"));
 			return 1;
 		}
+
 		close(infds[FDOUT]);
 		close(infds[FDIN]);
-
-		if (dup2(outfds[FDOUT], STDOUT_FILENO) < 0) {
-			TRACE(("leave sessioncommand: error redirecting stdout"));
-			return 1;
-		}
 		close(outfds[FDIN]);
 		close(outfds[FDOUT]);
-
-		if (dup2(errfds[FDOUT], STDERR_FILENO) < 0) {
-			TRACE(("leave sessioncommand: error redirecting stderr"));
-			return 1;
-		}
 		close(errfds[FDIN]);
 		close(errfds[FDOUT]);
 
@@ -582,13 +596,9 @@ static int noptycommand(struct Channel *channel, struct ChanSess *chansess) {
 		ses.maxfd = MAX(ses.maxfd, channel->outfd);
 		ses.maxfd = MAX(ses.maxfd, channel->errfd);
 
-		if (fcntl(channel->outfd, F_SETFL, O_NONBLOCK) < 0) {
-			dropbear_exit("Couldn't set nonblocking");
-		}
-		if (fcntl(channel->infd, F_SETFL, O_NONBLOCK) < 0) {
-			dropbear_exit("Couldn't set nonblocking");
-		}
-		if (fcntl(channel->errfd, F_SETFL, O_NONBLOCK) < 0) {
+		if ((fcntl(channel->outfd, F_SETFL, O_NONBLOCK) < 0) ||
+			(fcntl(channel->infd, F_SETFL, O_NONBLOCK) < 0) ||
+			(fcntl(channel->errfd, F_SETFL, O_NONBLOCK) < 0)) {
 			dropbear_exit("Couldn't set nonblocking");
 		}
 	}
@@ -604,7 +614,6 @@ static int ptycommand(struct Channel *channel, struct ChanSess *chansess) {
 
 	pid_t pid;
 	struct logininfo *li;
-	
 
 	TRACE(("enter ptycommand"));
 
@@ -622,18 +631,10 @@ static int ptycommand(struct Channel *channel, struct ChanSess *chansess) {
 
 		pty_make_controlling_tty(&chansess->slave, chansess->tty);
 		
-		if (dup2(chansess->slave, STDIN_FILENO) < 0) {
-			TRACE(("leave sessioncommand: error redirecting stdin"));
-			return 1;
-		}
-
-		if (dup2(chansess->slave, STDOUT_FILENO) < 0) {
-			TRACE(("leave sessioncommand: error redirecting stdout"));
-			return 1;
-		}
-
-		if (dup2(chansess->slave, STDERR_FILENO) < 0) {
-			TRACE(("leave sessioncommand: error redirecting stderr"));
+		if ((dup2(chansess->slave, STDIN_FILENO) < 0) ||
+			(dup2(chansess->slave, STDERR_FILENO) < 0) ||
+			(dup2(chansess->slave, STDOUT_FILENO) < 0)) {
+			TRACE(("leave sessioncommand: error redirecting filedesc"));
 			return 1;
 		}
 
@@ -642,7 +643,7 @@ static int ptycommand(struct Channel *channel, struct ChanSess *chansess) {
 		/* write the utmp/wtmp login record - must be after changing the
 		 * terminal used for stdout with the dup2 above */
 		li= login_alloc_entry(getpid(), ses.authstate.username,
-				ses.addrstring, chansess->tty);
+				ses.hostname, chansess->tty);
 		login_login(li);
 		login_free_entry(li);
 
@@ -695,9 +696,8 @@ static void addchildpid(struct ChanSess *chansess, pid_t pid) {
 
 }
 
-
-
-
+/* Clean up, drop to user privileges, set up the environment and execute
+ * the command/shell. This function does not return. */
 static void execchild(struct ChanSess *chansess) {
 
 	char *argv[4];
@@ -708,13 +708,13 @@ static void execchild(struct ChanSess *chansess) {
 	/* wipe the hostkey */
 	sign_key_free(ses.opts->hostkey);
 
-	/* clear the state of the prng */
+	/* overwrite the prng state */
 	seedrandom();
 
 	/* close file descriptors except stdin/stdout/stderr
 	 * Need to be sure FDs are closed here to avoid reading files as root */
 	for (i = 3; i < ses.maxfd; i++) {
-		if (m_close(i) == -1) {
+		if (m_close(i) == DROPBEAR_FAILURE) {
 			dropbear_exit("Error closing file desc");
 		}
 	}
@@ -734,13 +734,10 @@ static void execchild(struct ChanSess *chansess) {
 	/* We can only change uid/gid as root ... */
 	if (getuid() == 0) {
 
-		if (setgid(ses.authstate.pw->pw_gid) < 0
-			|| initgroups(ses.authstate.pw->pw_name,
-				ses.authstate.pw->pw_gid) < 0) {
-			dropbear_exit("error changing user");
-		}
-
-		if (setuid(ses.authstate.pw->pw_uid) < 0) {
+		if ((setgid(ses.authstate.pw->pw_gid) < 0) ||
+			(initgroups(ses.authstate.pw->pw_name, 
+						ses.authstate.pw->pw_gid) < 0) ||
+			(setuid(ses.authstate.pw->pw_uid) < 0)) {
 			dropbear_exit("error changing user");
 		}
 	} else {
@@ -770,6 +767,15 @@ static void execchild(struct ChanSess *chansess) {
 		dropbear_exit("error changing directory");
 	}
 
+#ifndef DISABLE_X11FWD
+	/* set up X11 forwarding if enabled */
+	x11setauth(chansess);
+#endif
+#ifndef DISABLE_AGENTFWD
+	/* set up agent env variable */
+	agentset(chansess);
+#endif
+
 	/* an empty shell should be interpreted as "/bin/sh" */
 	if (ses.authstate.pw->pw_shell[0] == '\0') {
 		usershell = "/bin/sh";
@@ -803,7 +809,8 @@ static void execchild(struct ChanSess *chansess) {
 	dropbear_exit("child failed");
 }
 	
-static void addnewvar(const char* param, const char* var) {
+/* add a new environment variable, allocating space for the entry */
+void addnewvar(const char* param, const char* var) {
 
 	char* newvar;
 	int plen, vlen;
